package dataspork.reader;
import dataspork.tree.DataSource;
import dataspork.util.SciStreamTokenizer;
import java.io.*;
import java.net.*;
import java.util.Vector;

public class MultipleConfigSource implements DataSource {

  public String filename;
  public URL url;
  public double[] data;
  public double[] boxlen;
  public int cols;           // Number of physical dimensions (columns of data)
  public int npart;          // Number of particles
  public int nconfig;        // Number of configurations

  public MultipleConfigSource(URL url) {
 //     this.filename = file;
    this.url=url;
      this.cols = 3;
      this.boxlen = new double[3];
       boxlen[0] = 0.0; boxlen[1] = 0.0; boxlen[2] = 0.0;
      this.npart = 0;
      this.nconfig = 0;
  }

  public Object getData() {
    if (data==null) readData();
    return data;
  }

  public int[] getDims() {
    if (data==null) readData();
    return new int[] {cols,npart,nconfig};
  }

  public double[] getBoxlen(){
    if (data==null) readData();
    return boxlen;
  }

  protected void readData() {
    if (data==null) data=new double[100];
    try { 
  //     String filename = url.getFile();
   //     filename = "test.dat";
   //     System.out.println("Filename: "+filename+"\n");
   //   BufferedReader reader = new BufferedReader(new FileReader(filename));
      BufferedReader reader = new BufferedReader(
                               new InputStreamReader(url.openStream()));
      StreamTokenizer st = new SciStreamTokenizer(reader);
      st.eolIsSignificant(true);
    //  st.commentChar('#');
    //  st.wordChars('#','#');
      int line=-1;
      int token;
      boolean do_preamble=true;

  // First read the preamble

      while (do_preamble){
        token = st.nextToken();
        if (st.ttype=='#') {
           token = st.nextToken();
         //  System.out.println("Word: "+st.toString()+"\n");
           if (token==st.TT_EOL) continue;
           if (st.sval.equalsIgnoreCase("boxlen")) {
         //     System.out.println("BoxLen: "+st.toString()+"\n");
                 Vector bl = new Vector();
                 token = st.nextToken();
                  bl.add(new Double(st.nval));
                 token = st.nextToken();
                  bl.add(new Double(st.nval));
                 token = st.nextToken();
                 if (st.ttype!=st.TT_EOL) {
                     bl.add(new Double(st.nval));
                 }
                  boxlen = new double[bl.size()];
                  for (int i = 0;i<bl.size();i++) {
                     boxlen[i] = ((Double)bl.elementAt(i)).doubleValue();
                  }
           }
           while (st.nextToken()!=st.TT_EOL);
        } else {
           if (st.ttype!=st.TT_EOL) {
               do_preamble=false;
               st.pushBack();
           }
        }
      }

  // Preamble is done, read the first line of data to determine # of dimensions
      st.commentChar('#');
      cols = 0;
      while ( (token=st.nextToken())!=st.TT_EOL) {
          cols++;
          line++;
          data[line]=st.nval;
      }
      npart++;

 // Determined # of dimensions, now determine # of particles

      while ( (token=st.nextToken())!=st.TT_EOF) {
        if (st.ttype==st.TT_EOL) {break;}
        for (int j=0; j<cols; j++)  {
           line++;
           if (line>=data.length) { //allocate larger array.
             double[] newData = new double[data.length+100];
             for (int i=0; i<data.length; ++i) newData[i]=data[i];
             data = newData;
           }
           data[line]=st.nval;
           st.nextToken();
        }
        npart++;
      }
      nconfig++;

 // Now read the rest of data
     boolean more_stuff=true;
     boolean frame_valid =true;
     if (st.nextToken()==st.TT_EOF) more_stuff=false;
     while (more_stuff) {
       for (int i=0;i<npart;i++){
          for (int j =0;j<cols;j++){
             line++;
             if (line>=data.length) { //allocate larger array.
                double[] newData = new double[data.length+100];
                for (int k=0; k<data.length; ++k) newData[k]=data[k];
                data = newData;
             }
             data[line]=st.nval;
             if (st.nextToken()==st.TT_EOF) {
                 more_stuff=false;
                 frame_valid=false;
                 break;
             }
          }
          if (st.nextToken()==st.TT_EOF) {
              more_stuff=false;
          }
       }
       if (st.nextToken()==st.TT_EOF) {
           more_stuff=false;
       }
       if (frame_valid) nconfig++;
     }

 // All the data is read.  Put it in a just-right sized array.
      double[] resize = new double[line+1];
      for (int i=0; i<resize.length; ++i) resize[i]=data[i];
      data=resize;
    }
    catch (EOFException e) {}
    catch (IOException e) {data=null;}
    System.out.println("Dimensions: "+cols+" Npart: "+npart+" Nconfig: "
                        +nconfig+"\n");
  }

  public int getType() {
    return DOUBLE;
  }
}
