package dataspork.stat;

import dataspork.util.grModel;
import dataspork.util.DataListener;
import dataspork.util.DoubleWindowModel;
import javax.swing.event.EventListenerList;
import java.io.*;
import java.text.DecimalFormat;

/** Computes the pair correlation function, g(r), from configuration data.
  */
public class grData implements DataListener{

   protected int dims[];
   protected double config[];
   protected double boxsize[];

   protected double rgrid[];
   protected double gr[];

   protected double grmax;


   protected grModel grmodel;

   protected DoubleWindowModel box_window;

   protected EventListenerList listenerList;


   public grData(int[] dims, double[] config, double[] boxsize){
      this.config = config;
      this.dims = dims;
      this.boxsize = boxsize;
      listenerList = new EventListenerList();
    //  config_window = new DoubleWindowModel(0,dims[2],1);
      grmodel = new grModel(.1,0,dims[2]-1);
      double rmaxsq=0.;
      for (int i=0;i<boxsize.length;i++) {
           if (rmaxsq<boxsize[i]/2.0) {rmaxsq = boxsize[i]/2.0;} 
   //       rmaxsq += boxsize[i]*boxsize[i];
      }
     // box_window = new DoubleWindowModel(0,Math.sqrt(rmaxsq),0);
      box_window = new DoubleWindowModel(0,rmaxsq,0);
      grmax = 0.0;
      calcGr();
      box_window.addListener(this);
      grmodel.addListener(this);
   }

   public void calcGr(){
         double dr  = grmodel.getdr();

         int start = grmodel.getCutoff();
         double low = box_window.getLow();
         double high = box_window.getHigh();

         Double dist = new Double( (high-low)/dr);
         int nbins =  dist.intValue() + 1;

         rgrid = new double[nbins];
         for (int i=0;i<nbins;i++){
              rgrid[i] = i*dr + .5*dr;
         }

         int[] grbins = new int[nbins];
         gr = new double[nbins];

         int dim = dims[0];
         int npart = dims[1];
         int nconfig = dims[2];
         int indx;
         double sum,r;
         double[] rd = new double[dim];
         double[] bx2 = new double[dim];
         for (int d=0;d<dim;d++){
           bx2[d] = boxsize[d]/2.0;
         }
         
         
         {int i,j,d;
         for (int nc=start;nc<nconfig;nc++){
             for (i=0;i<npart;i++){
                for (j=0;j<i;j++){
                   sum =0.0;
                   for (d=0;d<dim;d++){
                     rd[d] = config[nc*npart*dim + j*dim + d]
                        - config[nc*npart*dim + i*dim + d];
                      rd[d] = Math.abs( Math.abs(rd[d])-bx2[d]) - bx2[d];
                     sum += rd[d]*rd[d];
                   }
                   r = Math.sqrt(sum);
                   dist = new Double((r-low)/dr);
                   indx = dist.intValue();
                   if (indx<nbins) {
                      grbins[indx] += 2;
                   }
                   
                }
             }

         }
         }
         double[] sa = new double[3];
         sa[0] = 2.0;
         sa[1] = Math.PI;
         sa[2] = Math.PI*4.0/3.0;
         double norm,dv,vol,cst,vlast,deltav;
         vol = 1.0;
         for (int d=0;d<dim;d++){
           vol *= boxsize[d];
         }
         cst = vol/(npart*npart*(nconfig-start));
        // System.out.println("Vol: "+vol+"  cst:"+cst+"\n");
         vlast=0.0;
         for (int i=0;i<nbins;i++){
              dv = sa[dim-1]*Math.pow(((i+1)*dr),dim);
              deltav = dv-vlast;
              gr[i] = cst*grbins[i]/deltav;
              if (gr[i]>grmax) {grmax = gr[i];}
              vlast = dv;
            
         }
       ////  System.out.println("gr[0]:"+gr[0]+"  gr[1]:"+gr[1]+"\n");
       //  for (int i=0;i<nbins;i++){
       //       gr[i] = 1+i;
       //       if (gr[i]>grmax) {grmax = gr[i];}
       //  }
   }

   public double[] getGr(){
      return gr;
   }

   public double[] getRGrid(){
      return rgrid;
   }

   public grModel getgrModel(){
      return grmodel;
   }

   public double grMax(){
      return grmax;
   }

   public void addListener(DataListener e){
      listenerList.add(DataListener.class,e);
   }

   protected void fireListeners(Object o) {
     Object[] listeners = listenerList.getListenerList();
     for (int i = listeners.length-2; i>=0; i-=2) {
       if (o!=listeners[i+1]) {
         ((DataListener)listeners[i+1]).dataChanged(this);
       }
     }
   }

   public void dataChanged(Object o) {
     calcGr();
     fireListeners(o);
   }

   public void write(Writer w) throws IOException {
      DecimalFormat rf = new DecimalFormat("#0.0000");
      DecimalFormat gf = new DecimalFormat("0.0000E00");
      for (int i=0;i<rgrid.length;i++){
       String s =  new String(rf.format(rgrid[i]) +"  "+gf.format(gr[i])+" \n");
        w.write(s);
      }
   }

}

