package ncsa.hdf.jhv;

import java.awt.*;
import java.awt.event.*;

import ncsa.hdf.awt.Slider;
import ncsa.hdf.awt.MultiLineLabel;
import ncsa.hdf.awt.OutlinePanel;
import ncsa.hdf.awt.event.*;

// Select 3-D SDS slice value
public class SliceSelection
    extends Dialog
    implements ActionListener, SliderListener, ItemListener
{
    int activeSlice, x, y, z;
    HDFTreeNode node;
    Choice xChoice, yChoice, zChoice;
    TextField sliceField, startField, endField;
    Slider selectedSliceSlider;
    SDSDimInfo sliceInfo;
    Button okButton;

    // Inner class for Window closing
    class SliceSelectionClosedProcess
        extends WindowAdapter
    {
        public void windowClosing(WindowEvent evt)
        { dispose(); }
    }

    public SliceSelection(Frame parent, HDFTreeNode node, SDSDimInfo slice, String[] items)
    {
        super(parent, "SDS Dimension and Slice Selection", true);

        this.sliceInfo = slice;
        this.node = node;

        addNotify();
        addWindowListener(new SliceSelectionClosedProcess() );
        setBackground(Color.lightGray);

        // set up diemension choices and display them
        OutlinePanel lPanel = new OutlinePanel("Dimension Selection");
        Panel lPanel1 = new Panel();
        lPanel1.setLayout(new GridLayout(3, 2, 3, 3));
        xChoice = new Choice();
        yChoice = new Choice();
        zChoice = new Choice();
        String theItem = "";
        if ( items != null && items.length > 2)
        {
            for (int i=0; i<items.length; i++)
            {
                theItem = items[i];
                if (theItem==null || theItem.length()<1)
                    theItem = "fakeDim"+i;
                xChoice.add(theItem);
                yChoice.add(theItem);
                zChoice.add(theItem);
            }
            x = sliceInfo.getSelected(0);
            y = sliceInfo.getSelected(1);;
            z = sliceInfo.getSelected(2);;
            xChoice.select(x);
            yChoice.select(y);
            zChoice.select(z);
        }
        lPanel1.add(new Label("Horizontal:"));
        lPanel1.add(xChoice);
        lPanel1.add(new Label("Vertical:"));
        lPanel1.add(yChoice);
        lPanel1.add(new Label("Slice axis:"));
        lPanel1.add(zChoice);
        xChoice.addItemListener(this);
        yChoice.addItemListener(this);
        zChoice.addItemListener(this);

        Panel lPanel2  = new Panel();
        lPanel2.add(new Label("2D planes along the slice axis"));
        lPanel.setLayout(new BorderLayout());
        lPanel.add("North", lPanel1);
        lPanel.add("West", lPanel2);

        // place selection field and slider
        OutlinePanel rPanel = new OutlinePanel("Slice Selection");
        Panel rPanel1 = new Panel();
        rPanel1.setLayout(new GridLayout(1, 2));
        startField = new TextField("0");
        rPanel1.add(new Label("Starting slice:"));
        rPanel1.add(startField);

        Panel rPanel2 = new Panel();
        endField = new TextField("0");
        rPanel2.setLayout(new GridLayout(1, 2));
        rPanel2.add(new Label("Ending slice:"));
        rPanel2.add(endField);

        Panel rPanel3 = new Panel();
        sliceField = new TextField("0");
        rPanel3.setLayout(new GridLayout(1, 2));
        rPanel3.add(new Label("Current slice:"));
        rPanel3.add(sliceField);

        selectedSliceSlider = new Slider();
        selectedSliceSlider.setDispMode(Slider.SLIDERVALUE);

        startField.addActionListener(this);
        endField.addActionListener(this);
        sliceField.addActionListener(this);
        selectedSliceSlider.addSliderListener(this);

        rPanel.setLayout(new GridLayout(4,1,3,3));
        rPanel.add(rPanel1);
        rPanel.add(rPanel2);
        rPanel.add(rPanel3);
        rPanel.add(selectedSliceSlider);

        // place all components in the frame
        Panel panel = new Panel();
        panel.setLayout(new GridLayout(1, 2,3,3));
        panel.add(lPanel);
        panel.add(rPanel);

        setLayout(new BorderLayout());
        add("Center", panel);
        okButton = new Button("    Ok    ");
        okButton.addActionListener(this);
        Panel p = new Panel();
        p.add(okButton);
        add("South", p);

        int sliceIndex = sliceInfo.getSelected(2);
        int max = sliceInfo.getSize(sliceIndex)-1;
        selectedSliceSlider.setSliderRange(0,(double)max);
        selectedSliceSlider.setSliderValue(0);
        endField.setText(String.valueOf(max));

        pack();
        setResizable(false);
    }

    /**
     * SliderListener stuff
     */
    public void sliderStateChanged(SliderEvent evt)
    {
        Object target = evt.getSource();
        if (target instanceof Slider)
            handleSliderEvent((Slider)target);
    }

    /**
     * java.awt.event.ActionListener stuff
     */
    public void actionPerformed(ActionEvent evt)
    {
        Object target = evt.getSource();
        String arg = evt.getActionCommand();

        if(target instanceof TextField)
        {
            TextField textField = (TextField)target;
            handleTextFieldEvent(textField);
        }
        else if (target.equals(okButton))
        {
            dispose();
        }
    }

    /**
     * handles the Choice events
     */
    public void itemStateChanged(ItemEvent e)
    {
        Object source = e.getSource();
        boolean dimensionChanged = true;
        int selected = 0;

        if (source.equals(xChoice))
        {
            selected = xChoice.getSelectedIndex();
            if (selected == y) y = x;
            else if (selected == z) z = x;
            else if (selected == x) dimensionChanged = false;
            x = selected;
        }
        else if (source.equals(yChoice))
        {
            selected = yChoice.getSelectedIndex();
            if (selected == x) x = y;
            else if (selected == z) z = y;
            else if (selected == y) dimensionChanged = false;
            y = selected;
        }
        else if (source.equals(zChoice))
        {
            selected = zChoice.getSelectedIndex();
            if (selected == x) x = z;
            else if (selected == y) y = z;
            else if (selected == z) dimensionChanged = false;
            z = selected;
        }

        if (dimensionChanged)
        {
            xChoice.select(x);
            yChoice.select(y);
            zChoice.select(z);
            dimensionChanged();
        }
    }


    // update image by slice number
    public void handleTextFieldEvent(TextField target)
    {
        int selection = Integer.parseInt(target.getText());
        int sliceIndex = sliceInfo.getSelected(2);
        int max = sliceInfo.getSize(sliceIndex)-1;

        if (selection > max) selection = max;
        else if (selection < 0) selection = 0;

        if (target.equals(startField))
        {
            int end = sliceInfo.getEnd(2);
            if (selection > end) selection = end;
            sliceInfo.setStart(2, selection);
        }
        else if (target.equals(endField))
        {
            int start = sliceInfo.getStart(2);
            if (selection < start) selection = start;
            sliceInfo.setEnd(2, selection);
        }
        else if (target.equals(sliceField))
        {
            sliceInfo.setPlane(2, selection);
            selectedSliceSlider.setSliderValue((double)selection);

            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            node.getHdfSDSimage(getSliceInfo());
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }

        target.setText(String.valueOf(selection));

    }

    public void dimensionChanged()
    {
        //System.out.println("SliceSelection.dimensionChanged()");

        // update slice range
        int max = sliceInfo.getSize(z)-1;
        selectedSliceSlider.setSliderRange(0,(double)max);

        // update slice information
        sliceInfo.setSelected(x, y, z);
        sliceInfo.setStart(0, 0, 0);
        sliceInfo.setEnd(sliceInfo.getSize(x)-1, sliceInfo.getSize(y)-1, sliceInfo.getSize(z)-1);
        sliceInfo.setPlane(0, 0, 0);

        // update display fields
        startField.setText("0");
        endField.setText(String.valueOf(max));
        sliceField.setText("0");
        selectedSliceSlider.setSliderValue(0);

        // update image since slice info. has changed
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        node.getHdfSDSimage(getSliceInfo());
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    public void handleSliderEvent(Slider target)
    {
        // update slice selected value
        if (!selectedSliceSlider.sliderChanged())
            return;

        int selection = (int)Math.rint((double)selectedSliceSlider.getSliderValue());
        sliceField.setText(Integer.toString(selection));
        sliceInfo.setPlane(2, selection);

        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        node.getHdfSDSimage(getSliceInfo());
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    // get selected slice
    int getSelectedSlice()
    {
        return zChoice.getSelectedIndex();
    }

    public SDSDimInfo getSliceInfo()
    {
        return sliceInfo;
    }

    /** overwirte Dialog */
    public void dispose()
    {
        super.dispose();
        handleTextFieldEvent(startField);
        handleTextFieldEvent(endField);
        node.sliceSelection = null;
    }

}
