/****************************************************************************
 * NCSA HDF                                                                 *
 * National Comptational Science Alliance                                   *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING file.                                                        *
 *                                                                          *
 ****************************************************************************/

package ncsa.hdf.message;

import ncsa.hdf.hdflib.HDFLibrary;
import ncsa.hdf.util.Queue;

/**
 *  HDFHierarchy holds HDF Hierarchical tree information
 
 *  @version   1.1.3 September 22 1997
 *  @author    Peter X. Cao
 */
public class HDFHierarchy
    extends HDFObject
{
    /** the Queue containing the node information */
    private Queue nodeQueue;

    /**
     *  Constructs an HDFHierarchy with default parameters
     */
    public HDFHierarchy ()
    {
        super();
        nodeQueue = new Queue();
    }

    /**
     *  Construct an HDFHierarchy with specified HDFObjectNode and filename
     *
     *  @param filename the requested HDF file
     */
    public HDFHierarchy (String filename)
    {
        super(null, filename);
        nodeQueue = new Queue();
    }

    /**
     *  create a message for sending to the client
     *
     *  @return  The HDFMessage created by this object
     */
    public HDFMessage toClient()
    {
        HDFMessage message = new HDFMessage(getClass().getName());

        message.put("nodeQueue", nodeQueue);
        return message;
    }

    /**
     *  serve the client request on the server
     */
    public void service()
    {
        this.hdf =  new HDFLibrary();
        HDFAnalyse analyseHdf   = new HDFAnalyse();
        if (debug) analyseHdf.setDebug();
        analyseHdf.getHdfObject(filename, nodeQueue);
    }

    /**
     *  process a message receiving from the server
     *
     *  @param message  the HDFMessage received from the server
     */
    public void fromServer(HDFMessage message)
    {
        nodeQueue = (Queue) message.get("nodeQueue");
    }

    /** return the Queue containing the HDF hierarchy information */
    public Queue getQueue() { return nodeQueue; }

}
