package pupi.reader.restart;
import dataspork.tree.DataSource;
import java.util.*;
import ncsa.hdf.hdflib.*;

public class ScalarArraySource implements DataSource {

  protected int[] dims;
  protected String filename;
  protected HDFLibrary hdf;
  protected Vector idList;
  protected class DataID{
    int tag, ref, size;
    DataID(int tag, int ref, int size) {
      this.tag=tag; this.ref=ref; this.size=size;
    }
  }

  public ScalarArraySource(HDFParser.Element e) {
    dims = new int[] {0};
    filename=e.getFilename();
    hdf=e.getHDFLibrary();
    idList = new Vector();
  }

  public void addBlockSums(HDFParser.Element e, int blocksize) {
    dims[0]+=blocksize;
    idList.addElement(new DataID(e.tag,e.ref,blocksize)); 
  }

  public Object getData() {
    //Read the data from the HDF file.
    double[] data = new double[dims[0]];
    try {
      int fileID = hdf.Hopen(filename);
      hdf.Vstart(fileID);
      int iread=0;
      Enumeration e = idList.elements();
      while (e.hasMoreElements()) {
        DataID id = (DataID)e.nextElement();
        int length = hdf.Hlength(fileID,id.tag,id.ref);
        byte[] buf = new byte[length];
        hdf.Hgetelement(fileID,id.tag,id.ref,buf);
        byte[] buf2 = new byte[id.size*8];
        for (int i=0; i<id.size*8; i++) buf2[i]=buf[i+4];
        double[] temp = HDFNativeData.byteToDouble(buf2);
        for (int i=0; i<id.size; i++) data[i+iread]=temp[i];
        iread+=id.size;
      }
      hdf.Vend(fileID);
      hdf.Hclose(fileID);
    }
    catch (Throwable t) {t.printStackTrace();}
    return data;
  }

  public int[] getDims() {
    return dims;
  }

  public int getType() {
    return DOUBLE;
  }
}
